@@ -5,7 +5,7 @@ import click
 import pkg_resources
 from datetime import datetime
 from uuid import uuid4
-from flask import Response, redirect
+from flask import Response, redirect, abort
 from flask_compress import Compress
 from flask_cors import CORS
 from pymacaron.apiloader import load_api_models_and_endpoints
@@ -26,6 +26,14 @@ class modelpool():
 
     def __init__(self, name):
         self.api_name = name
+        self._model_names = []
+
+    def get_model_names(self):
+        return self._model_names
+
+    def add_model(self, model_name, model_class):
+        self._model_names.append(model_name)
+        setattr(self, model_name, model_class)
 
     def __getattr__(self, model_name):
         raise Exception(f'Either {self.api_name}.yaml has not been loaded or it does not define object {model_name}')
@@ -52,7 +60,7 @@ class apipool():
         if not hasattr(apipool, api_name):
             setattr(apipool, api_name, modelpool(api_name))
         models = getattr(apipool, api_name)
-        setattr(models, model_name, model_class)
+        models.add_model(model_name, model_class)
 
     @classmethod
     def get_model(cls, api_name):
@@ -66,7 +74,7 @@ class apipool():
         return sorted(apipool.__api_paths.keys())
 
     @classmethod
-    def load_swagger(cls, api_name, api_path, dest_dir=None, create_endpoints=False, force=False):
+    def load_swagger(cls, api_name, api_path, dest_dir=None, create_endpoints=False, force=False, model_file=None, app_file=None):
         """Load a swagger/openapi specification into pymacaron: generate its model
         classes (declared with pydantic), and optionally generate the Flask api
         endpoints binding endpoint methods to routes.
@@ -98,6 +106,8 @@ class apipool():
             dest_dir=dest_dir,
             create_endpoints=create_endpoints,
             force=force,
+            model_file=model_file,
+            app_file=app_file,
         )
 
         # Remember where this api's swagger is located
@@ -105,6 +115,42 @@ class apipool():
 
         return app_pkg
 
+    @classmethod
+    def enforce_global_model_names(cls):
+        """Find all models that have the same names in different apis and compare their
+        schema declarations.  Raise an error if at least two of them differ,
+        and show all the differences that were found as a diff of their json
+        schemas.
+        """
+
+        model_name_to_apis = {}
+
+        for api_name in apipool.__api_paths.keys():
+            api = getattr(apipool, api_name)
+            for model_name in api.get_model_names():
+                if model_name not in model_name_to_apis:
+                    model_name_to_apis[model_name] = []
+                model_name_to_apis[model_name].append(api)
+
+        found_names = []
+
+        for model_name in sorted(model_name_to_apis.keys()):
+            apis = model_name_to_apis[model_name]
+            cnt = len(apis)
+            if cnt > 1:
+                log.info(f"Found {model_name} in {cnt} apis: {', '.join([str(a) for a in apis])}")
+                m0 = getattr(apis[0], model_name)
+                for api in apis[1:]:
+                    m1 = getattr(api, model_name)
+                    diff = m0.diff_with(m1)
+                    if diff:
+                        log.error('\n' + diff)
+                        if model_name not in found_names:
+                            found_names.append(model_name)
+
+        if found_names:
+            raise Exception(f"Found models with same names but different schemas in different apis: {', '.join(found_names)}")
+
     @classmethod
     def publish_apis(cls, app, path='doc'):
         """Add routes to the Flask app to publish all loaded swagger files under the
@@ -114,40 +160,40 @@ class apipool():
         if not apipool.__api_paths:
             raise Exception("You must call .load_apis() before .publish_apis()")
 
-        # Infer the live host url from pym-config.yaml
-        proto = 'http'
-        if hasattr(get_config(), 'aws_cert_arn'):
-            proto = 'https'
+        conf = get_config()
 
-        live_host = "%s://%s" % (proto, get_config().live_host)
+        # Infer the live host url from pym-config.yaml
+        proto = 'https'
+        live_host = f"{proto}://{conf.live_host}"
 
         # Allow cross-origin calls
         CORS(app, resources={r"/%s/*" % path: {"origins": "*"}})
 
-        # Add routes to serve api specs and redirect to petstore ui for each one
-        for api_name, api_path in apipool.__api_paths.items():
+        def doc_endpoint(name=None):
+            api_name = name.replace('.yaml', '')
+            if api_name not in apipool.__api_paths:
+                log.info(f"Unknown api name '{api_name}'")
+                abort(404)
 
-            api_filename = os.path.basename(api_path)
-            log.info("Publishing api %s at /%s/%s" % (api_name, path, api_name))
+            api_path = apipool.__api_paths[api_name]
 
-            def redirect_to_petstore(live_host, api_filename):
-                def f():
-                    url = 'http://petstore.swagger.io/?url=%s/%s/%s' % (live_host, path, api_filename)
-                    log.info("Redirecting to %s" % url)
-                    return redirect(url, code=302)
-                return f
+            if name.endswith('.yaml'):
+                # Show the swagger file
+                with open(api_path, 'r') as f:
+                    spec = f.read()
+                    log.info("Returning %s" % api_path)
+                    return Response(spec, mimetype='text/plain')
 
-            def serve_api_spec(api_path):
-                def f():
-                    with open(api_path, 'r') as f:
-                        spec = f.read()
-                        log.info("Serving %s" % api_path)
-                        return Response(spec, mimetype='text/plain')
-                return f
+            else:
+                # Redirect to swagger-UI at petstore, to open this swagger file
+                url = f'http://petstore.swagger.io/?url={live_host}/{path}/{api_name}.yaml'
+                log.info(f"Redirecting to {url}")
+                return redirect(url, code=302)
 
-            # Publish yaml file and a redirect link to petstore's swagger UI to visualize that file
-            app.add_url_rule('/%s/%s' % (path, api_name), str(uuid4()), redirect_to_petstore(live_host, api_filename))
-            app.add_url_rule('/%s/%s' % (path, api_filename), str(uuid4()), serve_api_spec(api_path))
+        path = path.strip('/')
+        route = f'/{path}/<name>'
+        log.info(f"Publishing apis under route {route}")
+        app.add_url_rule(route, str(uuid4()), view_func=doc_endpoint, methods=['GET'])
 
 
 class jsonencoders:
@@ -293,11 +339,16 @@ class API(object):
         return self
 
 
-    def load_apis(self, path=None, ignore=[], only_models=[], force=False):
+    def load_apis(self, path=None, ignore=[], only_models=[], force=False, global_names=False):
         """Find all swagger files under the given path. Ignore those whose name is in
         the ignore list. Generate and load models for all others. Generate
         Flask app code for all except those in ignore and only_models list.
 
+        If global_names is true, load_apis() will check all generated models
+        from all loaded apis and if two models with the same name but different
+        schemas are found, raise an exception and show the diff between those
+        two models.
+
         """
 
         if not path:
@@ -344,6 +395,9 @@ class API(object):
             if app_pkg:
                 self.app_pkgs.append(app_pkg)
 
+        if global_names:
+            apipool.enforce_global_model_names()
+
         return self
 
 
